from pydantic import Field
from quart import Blueprint, g
from quart_schema import validate_response, validate_request
from sqlalchemy import select

from foxnouns.auth import require_auth
from foxnouns.db import Member
from foxnouns.db.aio import async_session
from foxnouns.db.util import user_from_ref, is_self
from foxnouns.exceptions import NotFoundError, ErrorCode
from foxnouns.models import BasePatchModel
from foxnouns.models.member import FullMemberModel
from foxnouns.models.fields import ProfileField, FieldEntry, PronounEntry
from foxnouns.settings import BASE_DOMAIN

bp = Blueprint("members_v2", __name__)


@bp.get("/api/v2/users/<user_ref>/members", host=BASE_DOMAIN)
@validate_response(list[FullMemberModel], 200)
async def get_members(user_ref: str):
    async with async_session() as session:
        user = await user_from_ref(session, user_ref)
        if not user:
            raise NotFoundError("User not found", type=ErrorCode.UserNotFound)

        return [FullMemberModel.model_validate(m) for m in user.members]


class MemberPostData(BasePatchModel):
    name: str = Field(min_length=1, max_length=100)  # TODO: validate member names more
    bio: str | None = Field(max_length=1024, default=None)

    names: list[FieldEntry] = Field(default=[])
    pronouns: list[PronounEntry] = Field(default=[])
    fields: list[ProfileField] = Field(default=[])


@bp.post("/api/v2/members", host=BASE_DOMAIN)
@require_auth(scope="member.create")
@validate_request(MemberPostData)
@validate_response(FullMemberModel, 200)
async def create_member(data: MemberPostData):
    async with async_session() as session:
        member = Member(
            user_id=g.user.id,
            name=data.name,
            bio=data.bio,
            names=[e.model_dump() for e in data.names],
            pronouns=[e.model_dump() for e in data.pronouns],
            fields=[e.model_dump() for e in data.fields],
        )

        session.add(member)
        await session.commit()
        await member.awaitable_attrs.user

        return FullMemberModel.model_validate(member)
