from pydantic import Field, field_validator
from quart import Blueprint, g
from quart_schema import validate_response, validate_request
from sqlalchemy import select

from foxnouns.auth import require_auth
from foxnouns.db import User
from foxnouns.db.aio import async_session
from foxnouns.db.util import user_from_ref, is_self
from foxnouns.exceptions import NotFoundError, ErrorCode
from foxnouns.models import BasePatchModel
from foxnouns.models.user import UserModel, SelfUserModel, check_username
from foxnouns.settings import BASE_DOMAIN

bp = Blueprint("users_v2", __name__)


@bp.get("/api/v2/users/<user_ref>", host=BASE_DOMAIN)
@validate_response(UserModel, 200)
async def get_user(user_ref: str):
    async with async_session() as session:
        user = await user_from_ref(session, user_ref)
        if not user:
            raise NotFoundError("User not found", type=ErrorCode.UserNotFound)

        return (
            SelfUserModel.model_validate(user)
            if is_self(user)
            else UserModel.model_validate(user)
        )


class EditUserRequest(BasePatchModel):
    username: str | None = Field(
        min_length=2, max_length=40, pattern=r"^[\w\-\.]{2,40}$", default=None
    )
    display_name: str | None = Field(max_length=100, default=None)
    bio: str | None = Field(max_length=1024, default=None)

    @field_validator("username")
    @classmethod
    def check_username(cls, value):
        return check_username(value)


@bp.patch("/api/v2/users/@me", host=BASE_DOMAIN)
@require_auth(scope="user.update")
@validate_request(EditUserRequest)
@validate_response(SelfUserModel, 200)
async def edit_user(data: EditUserRequest):
    async with async_session() as session:
        user = await session.scalar(select(User).where(User.id == g.user.id))

        if data.username:
            user.username = data.username
        if data.is_set("display_name"):
            user.display_name = data.display_name
        if data.is_set("bio"):
            user.bio = data.bio

        await session.commit()

        return SelfUserModel.model_validate(user)
